import requests
from typing import Set, Optional

from .base import API
from ..core import decorators
from ..core.models import Project


class APIProject(API):
    def __init__(self, token: str):
        super().__init__(token)

    @decorators.handle_http_errors
    def fetch_all(self, filter_keyword="active") -> Set[Project]:
        r = requests.get(
            "https://www.meistertask.com/api/projects",
            headers=self.headers,
            params={"status": str(filter_keyword).strip().lower()},
        )

        if r.status_code != 200:
            raise Exception(
                "Failed to connect to meistertask (code: {})".format(r.status_code)
            )

        return set(map(lambda project: Project.from_json(project), r.json()))

    @decorators.handle_http_errors
    def fetch_by_id(self, project_id: int) -> Project:
        r = requests.get(
            "https://www.meistertask.com/api/projects/{}".format(project_id),
            headers=self.headers,
        )

        if r.status_code != 200:
            raise Exception(
                "Failed to connect to meistertask (code: {})".format(r.status_code)
            )

        return Project.from_json(r.json())

    def fetch_by_name(self, name: str) -> Optional[Project]:
        projects: Set[Project] = self.fetch_all()
        for project in projects:
            if project.name == name:
                return project

        return None

    @decorators.handle_http_errors
    def create(self, project: Project) -> Project:
        r = requests.post(
            "https://www.meistertask.com/api/projects",
            headers=self.headers,
            data={"name": project.name, "notes": project.description},
        )

        if r.status_code != 200:
            raise Exception(
                "Failed to connect to meistertask (code: {})".format(r.status_code)
            )

        return Project.from_json(r.json())

    @decorators.handle_http_errors
    def update(self, project: Project) -> Project:

        r = requests.put(
            "https://www.meistertask.com/api/projects/{}".format(project.id),
            headers=self.headers,
            data={"name": project.name, "notes": project.description},
        )

        if r.status_code != 200:
            raise Exception(
                "Failed to connect to meistertask (code: {})".format(r.status_code)
            )

        return Project.from_json(r.json())

    @decorators.handle_http_errors
    def delete(self, project_id: int) -> Project:
        r = requests.put(
            "https://www.meistertask.com/api/projects/{}".format(project_id),
            headers=self.headers,
            data={"status": 4},
        )

        if r.status_code != 200:
            raise Exception(
                "Failed to connect to meistertask (code: {})".format(r.status_code)
            )

        return Project.from_json(r.json())

    @decorators.handle_http_errors
    def archive(self, project_id: int) -> Project:
        r = requests.put(
            "https://www.meistertask.com/api/projects/{}".format(project_id),
            headers=self.headers,
            data={"status": 5},
        )

        if r.status_code != 200:
            raise Exception(
                "Failed to connect to meistertask (code: {})".format(r.status_code)
            )

        return Project.from_json(r.json())
