import requests
from typing import Set

from .base import API
from ..core import decorators
from ..core.models import Task


class APITask(API):
    def __init__(self, token: str):
        super().__init__(token)

    @decorators.handle_http_errors
    def fetch_all(self, project_id: int, task_status="open") -> Set[Task]:
        r = requests.get(
            "https://www.meistertask.com/api/projects/{}/tasks".format(project_id),
            headers=self.headers,
            params={"status": task_status},
        )

        if r.status_code != 200:
            raise Exception(
                "Failed to connect to meistertask (code: {})".format(r.status_code)
            )

        return set(map(lambda task: Task.from_json(task), r.json()))

    @decorators.handle_http_errors
    def fetch_all_by_section(self, section_id: int, task_status="open"):
        r = requests.get(
            "https://www.meistertask.com/api/sections/{}/tasks".format(section_id),
            headers=self.headers,
            params={"status": task_status},
        )

        if r.status_code != 200:
            raise Exception(
                "Failed to connect to meistertask (code: {})".format(r.status_code)
            )

        return set(map(lambda task: Task.from_json(task), r.json()))

    @decorators.handle_http_errors
    def create(self, task: Task) -> Task:
        r = requests.post(
            "https://www.meistertask.com/api/sections/{}/tasks".format(task.section_id),
            headers=self.headers,
            data={"name": task.name, "notes": task.description},
        )

        if r.status_code != 200:
            raise Exception(
                "Failed to connect to meistertask (code: {})".format(r.status_code)
            )

        return Task.from_json(r.json())

    @decorators.handle_http_errors
    def update(self, task: Task) -> Task:
        r = requests.put(
            "https://www.meistertask.com/api/tasks/{}".format(task.id),
            headers=self.headers,
            data={"name": task.name, "notes": task.description},
        )

        if r.status_code != 200:
            raise Exception(
                "Failed to connect to meistertask (code: {})".format(r.status_code)
            )

        return Task.from_json(r.json())

    @decorators.handle_http_errors
    def move(self, task_id: int, section_id: int) -> Task:

        r = requests.put(
            "https://www.meistertask.com/api/tasks/{}".format(task_id),
            headers=self.headers,
            data={"section_id": section_id},
        )

        if r.status_code != 200:
            raise Exception(
                "Failed to connect to meistertask (code: {})".format(r.status_code)
            )

        return Task.from_json(r.json())
