from typing import Union, Set

from src.api import APITask, APISection
from src.core.errors import TaskError
from src.core.models import Task, Section, Project
from src.utils import Colors


class TaskManager:
    def __init__(self, token: str, project: Project):
        self.project = project
        self.api_section = APISection(token)
        self.api = APITask(token)

    def create(self, task_name: str, task_description: str, section_name: str):
        if len(task_name) < 3:
            raise TaskError("Task name must be at least three characters")

        section: Section = self.api_section.fetch_by_name(section_name, self.project.id)
        if not section:
            print(Colors.wrap(Colors.RED, "Section not found"))
            exit(145)

        task: Task = Task(
            name=task_name, description=task_description, project_id=self.project.id
        )
        task.section_id = section.id
        self.api.create(task)

        print(Colors.wrap(Colors.GREEN, "Task added successfully"))

    def update(self, task: Task):
        print(Task(task))

        name: str = str(
            input(Colors.wrap(Colors.YELLOW, "[?] Type name (Enter to skip):"))
        )
        if name and len(name):
            task.name = name

        description: str = str(
            input(Colors.wrap(Colors.YELLOW, "[?] Type description (Enter to skip): "))
        )
        if description and len(description):
            task.description = description

        self.api.update(task)
        print(Colors.wrap(Colors.GREEN, "Task updated successfully"))

    def move(self, task_id: int, section_id: int):
        task: Task = self.api.move(task_id, section_id)
        print(task)
        print(Colors.wrap(Colors.GREEN, "Task moved successfully"))

    def fetch_by_name(self, name: str) -> Union[Task, None]:
        tasks: Set[Task] = self.api.fetch_all(self.project.id)
        for task in tasks:
            if task.name == name:
                return task

        return None

    def list_all(self, section_name: str, task_status: str, listing_format="long"):
        tasks = self.__get_tasks_by_section(section_name, task_status)

        if tasks and len(tasks):
            for task in tasks:
                if listing_format == "short":
                    task.show_summary()
                else:
                    print(task)
            print(
                Colors.wrap(Colors.CYAN, "Total number of tasks {}".format(len(tasks)))
            )
        else:
            print(Colors.wrap(Colors.RED, "No task is found in this project"))

    def __get_tasks_by_section(self, section_name: str, task_status: str):
        """Get all tasks based on section name"""
        if section_name != "all":
            section: Section = self.api_section.fetch_by_name(
                section_name, self.project.id
            )
            if not section:
                print(Colors.wrap(Colors.RED, "Section not found"))
                exit(48)
            return self.api.fetch_all_by_section(section.id, task_status)
        else:
            return self.api.fetch_all(self.project.id, task_status)
